@@ -1,7 +1,7 @@
 # -*- coding: utf-8 -*-
 """
-    flaskext.lastuser
-    ~~~~~~~~~~~~~~~~~
+    flask.ext.lastuser
+    ~~~~~~~~~~~~~~~~~~
 
     Flask extension for Lastuser
 
@@ -16,6 +16,7 @@ import urlparse
 import requests
 import urllib
 import re
+import weakref
 from coaster.views import get_current_url, get_next_url
 
 from flask import session, g, redirect, url_for, request, flash, abort, Response
@@ -62,13 +63,46 @@ class UserManagerBase(object):
     """
     Base class for database-aware user managers.
     """
+    def load_user(self, userid, create=False):
+        raise NotImplementedError("Not implemented in the base class")
+
+    def make_userinfo(self, user):
+        raise NotImplementedError("Not implemented in the base class")
+
+    def load_user_userinfo(self, userinfo):
+        raise NotImplementedError("Not implemented in the base class")
+
     def before_request(self):
         """
         Listener that is called at the start of each request. Responsible for
         setting g.user and g.lastuserinfo
         """
-        g.user = None
-        g.lastuserinfo = None
+        if session.get('lastuser_userid'):
+            user = self.load_user(session['lastuser_userid'])
+            if user is None:
+                # We'll get here if one of two things happens:
+                # 1. This is a dev setup and the local database was deleted.
+                # 2. Multiple apps are hosted on one domain and we got a cookie
+                #    from another app, but we've never heard of this user before.
+                # In either situation, try to create a new user record
+                userdata = self.lastuser.getuser_by_userid(session['lastuser_userid'])
+                if userdata.get('type') == 'user':
+                    # This is an actual user. Make an account
+                    user = self.load_user(session['lastuser_userid'], create=True)
+                    user.username = userdata['name']
+                    user.fullname = userdata['title']
+                    self.db.session.commit()
+                else:
+                    # No such user. Pop the session
+                    session.pop('lastuser_userid', None)
+        else:
+            user = None
+
+        g.user = user
+        if user:
+            g.lastuserinfo = self.make_userinfo(user)
+        else:
+            g.lastuserinfo = None
 
     def login_listener(self, userinfo, token):
         """
@@ -77,11 +111,11 @@ class UserManagerBase(object):
         """
         self.before_request()
 
-    def user_emails(self, lastuser, user):
+    def user_emails(self, user):
         """
         Retrieve all known email addresses for the given user.
         """
-        result = lastuser.call_resource('email', all=1,
+        result = self.lastuser.call_resource('email', all=1,
             _token=user.lastuser_token,
             _token_type=user.lastuser_token_type)
 
@@ -123,6 +157,7 @@ class Lastuser(object):
         self.tokenverify_endpoint = app.config.get('LASTUSER_ENDPOINT_TOKENVERIFY', 'api/1/token/verify')
         self.getuser_endpoint = app.config.get('LASTUSER_ENDPOINT_GETUSER', 'api/1/user/get')
         self.getuser_userid_endpoint = app.config.get('LASTUSER_ENDPOINT_GETUSER_USERID', 'api/1/user/get_by_userid')
+        self.getorgteams_endpoint = app.config.get('LASTUSER_ENDPOINT_GETORGTEAMS', 'api/1/org/get_teams')
         self.client_id = app.config['LASTUSER_CLIENT_ID']
         self.client_secret = app.config['LASTUSER_CLIENT_SECRET']
 
@@ -134,6 +169,7 @@ class Lastuser(object):
 
     def init_usermanager(self, um):
         self.usermanager = um
+        um.lastuser = weakref.proxy(self)
 
     def before_request(self):
         if self.usermanager:
@@ -366,9 +402,14 @@ class Lastuser(object):
             return f(*args, **kw)
         return decorated_function
 
+    def endpoint_url(self, endpoint):
+        """
+        Returns the full URL to a given endpoint path on the current Lastuser server.
+        """
+        return urlparse.urljoin(self.lastuser_server, endpoint)
+
     def _lastuser_api_call(self, endpoint, **kwargs):
-        # Check this token with Lastuser's verify_token endpoint
-        r = requests.post(urlparse.urljoin(self.lastuser_server, endpoint),
+        r = requests.post(self.endpoint_url(endpoint),
             auth=(self.client_id, self.client_secret),
             data=kwargs)
         if r.status_code in (400, 500, 401):
@@ -410,7 +451,9 @@ class Lastuser(object):
                     return Response(u"Invalid token.", 403)
                 elif result['status'] == 'ok':
                     # All okay.
-                    # TODO: If the user is unknown, make a new user
+                    # If the user is unknown, make a new user. If the user is known, don't update scoped data
+                    g.user = self.usermanager.load_user_userinfo(result['userinfo'], update=False)
+                    g.lastuserinfo = self.usermanager.make_userinfo(g.user)
                     return f(result, *args, **kw)
             return decorated_function
         return inner
@@ -477,7 +520,17 @@ class Lastuser(object):
         Retrieve all known email addresses for the given user.
         """
         # TODO: If this is ever cached, provide a way to flush cache
-        return self.usermanager.user_emails(self, user)
+        return self.usermanager.user_emails(user)
+
+    def org_teams(self, org):
+        """
+        Get a list of teams in the given organization(s).
+        """
+        data = self._lastuser_api_call(self.getorgteams_endpoint, org=org)
+        if data['status'] == 'ok':
+            return data['org_teams']
+        else:
+            return {}
 
 # Compatibility name
 LastUser = Lastuser
